////////////////////////////////////////////////////////////////
//
// Copyright (C) 2006 Affymetrix, Inc.
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License 
// (version 2.1) as published by the Free Software Foundation.
// 
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
// for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
//
////////////////////////////////////////////////////////////////

/**
/// @file   apt-dump-pgf.cpp
/// @brief  Main for dumping pgf information.
*/

#include "DumpPgf.h"

using namespace std;

/**
\page apt-dump-pgf MANUAL: apt-dump-pgf (apt-1.8.0)

\p apt-dump-pgf will dump information from a pgf and clf file.

\section QuickStart Quick Start
To dump information from a pgf file, use:

\code
apt-dump-pgf -c file.clf -p file.pgf \
  [--probeset-type=type [--probeset-type=...]]

apt-dump-pgf -c file.clf -p file.pgf \
  [--probeset-ids=file [--probeset-ids=...]]

apt-dump-pgf -c file.clf -p file.pgf \
  [--probe-ids=file [--probe-ids=...]]

\endcode

\subsection manualOptions Options:

\verbatim
apt-dump-pgf - Dump information from a pgf file.
Usage:
   apt-dump-pgf -o int.txt -c file.clf -p file.pgf [--probeset-type=type [--probeset-type=...]]

   apt-dump-pgf -o int.txt -c file.clf -p file.pgf [--probeset-ids=file [--probeset-ids=...]]

   apt-dump-pgf -o int.txt -c file.clf -p file.pgf [--probe-ids=file [--probe-ids=...]]

options:
   -p, --pgf-file      The pgf file used to dump information. 
   -c, --clf-file      Optional clf file to use. When present, probe
                       position will be included in the output. 
       --probeset-type Optional probeset type to extract; can be
                       specified multiple times. When specified
                       multiple times, the intersection of all types
                       is taken. The user cannot mix use of
                       probeset-type, probeset-ids, and probe-ids. 
   -s, --probeset-ids  Optional name of a file containing probeset 
                       ids to extract; can be specified multiple
                       times. The user cannot mix use of
                       probeset-type, probeset-ids, and probe-ids. 
       --probe-ids     Optional name of a file containing probe ids 
                       to extract; can be specified multiple times.
                       The user cannot mix use of probeset-type,
                       probeset-ids, and probe-ids. 
       --probeset-only Dump only probeset level information. 
       --or            Use the union of the types requested, not the
                       intersection. 
   -o, --out-file      Output file to contain the dump output. 
       --version       Display version information. 
   -h, --help          Print help message. 
\endverbatim

*/

int main (int argc, char* argv[])
{
  const string version ("apt-1.8.0");
  const string cvsId ("$Id: apt-dump-pgf.cpp,v 1.12 2007/05/23 17:55:45 awilli Exp $");

  try
  {
    dumpPgf pgfDump (argc, argv, version, cvsId);
    pgfDump.run();
  }

  catch (exception& e)
  {
    // cerr << e.what() << endl; // errAbort() writes the error message.
    return 1;
  }
  return 0;
}
