////////////////////////////////////////////////////////////////
//
// Copyright (C) 2006 Affymetrix, Inc.
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License 
// (version 2.1) as published by the Free Software Foundation.
// 
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
// for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
//
////////////////////////////////////////////////////////////////


/**
 * @file   dump-pgf-test.cpp
 * @brief  Program for doing regression tests on dump-pgf.
 */
#include <vector>
#include <string>
#include "util/RegressionTest.h"
#include "util/TextFileCheck.h"
#include "util/Util.h"
#include "util/Verbose.h"

using namespace std;

class DumpPgfTest
{

public:
  int numPassed, numFailed;
  DumpPgfTest()
  {
    numPassed = 0;
    numFailed = 0;
  }
  void doDumpPgfTest();

private:
  string errorMsg;
};

void DumpPgfTest::doDumpPgfTest()
{
  // apt-dump-pgf does not make an output directory
  Util::createDir ("dump-pgf-test-generated");
  string command = "./apt-dump-pgf -out-file dump-pgf-test-generated/dump.probeset_ids_HG-U133_Plus_2.txt -p data/HG-U133_Plus_2.pgf -probeset-ids data/probeset_ids_HG-U133_Plus_2.txt";
  vector<RegressionCheck *> checks;
  RegressionTest dumpTest (command.c_str(), checks);
  Verbose::out (1, "Doing doDumpPgfTest()");
  if (!dumpTest.run())
  {
    Verbose::out (1, "Error in DumpPgfTest::doDumpPgfTest(): " + dumpTest.getErrorMsg());
    numFailed++;
  }
  else
  {
    TextFileCheck textCheck ("dump-pgf-test-generated/dump.probeset_ids_HG-U133_Plus_2.txt",
                             "data/dump.probeset_ids_HG-U133_Plus_2.txt", 5);
    if (textCheck.check (errorMsg))
    {
      numPassed++;
      Verbose::progressStep (1);
    }
    else
    {
      Verbose::out (1, "Error in DumpPgfTest::doDumpPgfTest(): " + errorMsg);
      numFailed++;
    }
  }
}

int main (int argc, char* argv[])
{
  DumpPgfTest test;
  Verbose::setLevel(2);
  Verbose::progressBegin (1, "Doing tests", 20, 1, 20);
  test.doDumpPgfTest();
  Verbose::progressEnd (1, "Done.");
  Verbose::out (1, "NumPassed: " + ToStr (test.numPassed) + " NumFailed: " + ToStr (test.numFailed));
  return test.numFailed != 0;
}
