\name{analysis.mmbgx}
\alias{analysis.mmbgx}
\alias{plotExpressionDensity}
\alias{plotDEDensity}
\alias{plotDEHistogram}
\alias{rankByDE}
\alias{plotDiffRank}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Analyse MMBGX output.}
\description{
  Functions for plotting expression densities, differential expression densities, histogram of proportion of differentially expressed transcripts, etc.
}
\usage{
  plotExpressionDensity(bgxOutput, probeset=NULL, ...)
  plotDEDensity(bgxOutput, probeset=NULL, conditions=c(1,2), ...)
  plotDEHistogram(bgxOutput, conditions=c(1,2), df=floor(1.8 * log10(length(bgxOutput[["PSids"]]))))
  rankByDE(bgxOutput, conditions=c(1,2), absolute=TRUE)
  plotDiffRank(bgxOutput, conditions=c(1,2), ymax=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{bgxOutput}{A list obtained from running \code{\link{readSingle.mmbgx}} on a MMBGX output directory.}
  \item{probeset}{Which probeset to analyse. This can either be an index or an ID.}
  \item{conditions}{Indices of conditions to compare.}
  \item{df}{Residual degrees of freedom. Decrease to 6 if the histogram fit goes haywire.}
  \item{absolute}{Rank probesets by absolute differential expression.}
  \item{ymax}{Specify upper limit of y axis.}
  \item{...}{Parameters to pass to density function (where applicable).}
}
\details{
  \code{plotExpressionDensity} plots expression distributions under various conditions for the specified probeset.

  \code{plotDEDensity} plots the differential expression distribution between two conditions for a given probeset.

  \code{plotDEHistogram} plots a histogram of differential expression between two conditions and estimates the number of up and down regulated differentially expressed probesetts.

  \code{rankByDE} ranks probesets by differential expression and returns ordering and corresponding DE values in a matrix.

  \code{plotDiffRank} plots 2.5-97.5\% confidence intervals for ranked differential expression estimates.
}
\value{
  None, except \code{rankByDE}, which returns a matrix of probesets ranked by differential expression.
}
\author{Ernest Turro}
\seealso{\code{\link{mmbgx}}, \code{\link{readSingle.mmbgx}}, \code{\link{combineRuns.mmbgx}}, \code{\link{plotExpressionDensity}}, \code{\link{plotDEDensity}}, \code{\link{plotDEHistogram}}}
%\examples{
%}
\keyword{IO}% at least one, from doc/KEYWORDS

