/*
 *  This file is part of BGX, the Bayesian Gene eXpression program.
 *  Copyright (c) 2008 Ernest Turro <ernest.turro@ic.ac.uk>
 *
 *  BGX is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *
 *  BGX is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 
#include <algorithm>
#include <map>
#include <ctype.h>
#include <cstdlib>
#include <cstring>
#ifdef WIN32
 #include <io.h>
#else
  #include <sys/stat.h>
#endif
#include <dirent.h>
#include <string>
#include "file/TsvFile/TsvFile.h"
#include "file/TsvFile/PgfFile.h"
#include <iostream>
#include <fstream>
#include <iomanip>

using namespace std;
using namespace affx;

int main(int argc, char **argv) {

  if(argc < 4) {
    cout << "Usage: exonstruct path_to_affy_pgf path_to_xmap_tsv output_directory\n";
    exit(1);
  }

  // Try opening Affy PGF file  
  PgfFile* pgf = new PgfFile();
  if ((pgf->open(string(argv[1])))!=TSV_OK) {
    printf("Error: couldn't open Affy PGF file \"%s\"\n",argv[1]);
    delete pgf;
    exit(1);
  }

  // Try opening X:Map TSV file
  TsvFile* tsv = new TsvFile();
  if ((tsv->open(string(argv[2])))!=TSV_OK) {
    printf("Error: couldn't open transcripthit TSV file \"%s\"\n",argv[2]);
    delete tsv;
    exit(1);
  }

  const char *struct_dir = argv[3];

  // Try opening the structure dir for BGX
  DIR* struct_dir_=opendir(struct_dir);
  char a = 'y';
  if(struct_dir_ != NULL){
    a = 'n';
    printf("Output dir \"%s\" already exists. Overwrite? (y/N) ", struct_dir);
    scanf("%c", &a);
    a = tolower(a);
  }
  if(a=='y') {
#ifdef WIN32
    mkdir(struct_dir);
#else
    mkdir(struct_dir,S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
#endif
  } else {
    exit(1);
  }
    
  map<const int, int> probeId2Index;
  vector<vector <int> > PS2PM;
  vector<vector <int> > PM2PS;
  vector<vector <int> > PS2PS;
  vector<vector <double> > PS2PSOverlap;
  vector<vector <int> > PMids;
  vector<int> PMcat;
  
  vector<int> BGcat;
  vector<int> BGids;

  vector<string> PSids;

  printf("Parsing PGF file... ");
  fflush(stdout);

  map<const string, vector <int> > seqToAffyIds;
  bool antigenomic=false;
  while(pgf->next_probeset()==TSV_OK){
    while(pgf->next_atom()==TSV_OK){
      while (pgf->next_probe()==TSV_OK) {
        seqToAffyIds[pgf->probe_sequence].push_back(pgf->probe_id);
        if(pgf->probeset_type=="control->bgp->antigenomic") {
          BGcat.push_back(pgf->gc_count);
          BGids.push_back(pgf->probe_id);
        }
      }
    }
  }
  printf("Done.\n");
  /*
  ofstream foor("seqToAffyIds.txt");
  map<const string, vector <int> >::iterator iterr;
  
  for(iterr = seqToAffyIds.begin(); iterr != seqToAffyIds.end(); iterr++) {
    foor << iterr->first << "\t";
    for(int i=0; i < (int)seqToAffyIds[iterr->first].size(); i++) {
      foor << seqToAffyIds[iterr->first][i] << " ";
    }
    foor << endl;
  }
  foor.close();
  */
//  ofstream seqs("sequences.txt");


  printf("Parsing TSV file... ");
  fflush(stdout);


  // Make bindings to TSV slots
  int transcript_id, probe_id, gc_count;
  string stable_id, probe_sequence;
  tsv->bind(0,"transcript_id",&transcript_id,TSV_BIND_REQUIRED);
  tsv->bind(0,"stable_id",&stable_id,TSV_BIND_REQUIRED);
  tsv->bind(1,"probe_id",&probe_id,TSV_BIND_REQUIRED);
  tsv->bind(1,"probe_sequence",&probe_sequence,TSV_BIND_REQUIRED);
  tsv->bind(1,"gc_count",&gc_count,TSV_BIND_REQUIRED);

  int j=0; // unique probe counter
  int g=0; // main gene counter

  while(tsv->nextLevel(0)==TSV_OK){
      PSids.push_back(stable_id);
      vector<int> pr;
      while (tsv->nextLevel(1)==TSV_OK) {
        if(probeId2Index.count(probe_id)==0) {
   //       seqs << tsv->probe_sequence << endl;
          probeId2Index[probe_id] = j;
          PM2PS.resize(j+1);
          PM2PS[j].push_back(g);
          pr.push_back(j);
          PMids.push_back(seqToAffyIds[probe_sequence]);
 //         PMids.push_back(tsv->probe_id);
          PMcat.push_back(gc_count);
          j++;
        } else {
          PM2PS[probeId2Index[probe_id]].push_back(g);
          pr.push_back(probeId2Index[probe_id]);
        }
      }
      PS2PM.push_back(pr);
      //cout << g << '\r' << flush;
      g++;
  }

  for(int g=0; g < (int)PS2PM.size(); g++) {
    vector<int> ge;
    vector<double> geOverlap;
    for(int p=0; p < (int)PS2PM[g].size(); p++) {
      for(int h=0; h < (int)PM2PS[PS2PM[g][p]].size(); h++) {
        if(find(ge.begin(), ge.end(), PM2PS[PS2PM[g][p]][h]) == ge.end()) {
          ge.push_back(PM2PS[PS2PM[g][p]][h]);
          vector<int> v1 = PS2PM[g], v2 = PS2PM[PM2PS[PS2PM[g][p]][h]];
          vector<int> res(PS2PM[g].size() + PS2PM[PM2PS[PS2PM[g][p]][h]].size()); 
          vector<int>::iterator it;
          sort(v1.begin(), v1.end()); sort(v2.begin(), v2.end());
          it = set_intersection(v1.begin(), v1.end(), v2.begin(), v2.end(), res.begin());
          geOverlap.push_back((double)(it - res.begin())/PS2PM[g].size());
        }
      }
    }
    PS2PS.push_back(ge);
    PS2PSOverlap.push_back(geOverlap);
  }


  printf("Done.\n");

  printf("Found %d probesets, %d unique probes.\n", (int)PS2PM.size(), (int)PM2PS.size());

  printf("Generating structure files...");
  fflush(stdout);

  ofstream of((string(struct_dir) + '/' + "PM2PS.txt").c_str());
  for(int i=0; i < (int)PM2PS.size(); i++) {
    for(int j=0; j < (int)PM2PS[i].size(); j++) {
      of << PM2PS[i][j] << " ";
    }
    of << endl;
  }
  of.close();
  of.open((string(struct_dir) + '/' + "PS2PM.txt").c_str());
  for(int i=0; i < (int)PS2PM.size(); i++) {
    for(int j=0; j < (int)PS2PM[i].size(); j++) {
      of << PS2PM[i][j] << " ";
    }
    of << endl;
  }
  of.close();
  of.open((string(struct_dir) + '/' + "PMids.txt").c_str());
  for(int i=0; i < (int)PMids.size(); i++) {
    for(int j=0; j < (int)PMids[i].size(); j++) {
      of << PMids[i][j] << "\t";
    }
    of << endl;
  }
  of.close();
  of.open((string(struct_dir) + '/' + "PSids.txt").c_str());
  for(int i=0; i < (int)PSids.size(); i++) of << PSids[i] << endl;
  of.close();
  of.open((string(struct_dir) + '/' + "BGids.txt").c_str());
  for(int i=0; i < (int)BGids.size(); i++) of << BGids[i] << endl;
  of.close();
  of.open((string(struct_dir) + '/' + "BGcats.txt").c_str());
  for(int i=0; i < (int)BGcat.size(); i++) of << BGcat[i] << endl;
  of.close();
  of.open((string(struct_dir) + '/' + "PMcats.txt").c_str());
  for(int i=0; i < (int)PMcat.size(); i++) of << PMcat[i] << endl;
  of.close();
  of.open((string(struct_dir) + '/' + "PS2PS.txt").c_str());
  for(int i=0; i < (int)PS2PS.size(); i++) {
    for(int j=0; j < (int)PS2PS[i].size(); j++) {
      of << PS2PS[i][j] << "(" << setprecision(4) << PS2PSOverlap[i][j] << ")\t";
    }
    of << endl;
  }
  of.close();

  printf("Done.\n");
  printf("Output in \"%s\".\n", struct_dir);

  tsv->close();
  delete tsv;

  return 0;
}
 
